from django.conf.urls import patterns
from django.conf.urls import url

from django_relinking import views

urlpatterns = patterns(
    '',
    url(r'^get_content_type_objects/$', views.get_objects, name="get_content_type_objects"),
)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django_relinking.content_types import get_object


class Link(models.Model):

    TARGET_CHOICES = [
        (0, "_self"),
        (1, "_blank")
    ]

    keys = models.TextField(
        verbose_name=_(u"keys for replacing"),
        help_text=_(u"One key per line. The key can include many words and different characters")
    )

    content_type = models.CharField(
        verbose_name=_(u"content type"),
        max_length=255,
        default=None,
        null=True,
        blank=True,
        choices=[]
    )
    object_pk = models.CharField(
        verbose_name=_(u"object"),
        max_length=255,
        default=None,
        null=True,
        blank=True
    )

    direct_link = models.CharField(
        verbose_name=_(u"direct link"),
        max_length=255,
        default=None,
        null=True,
        blank=True,
        help_text=_(u"If you set this field, 'content type' and 'object primary key' will not be used")
    )

    target = models.IntegerField(
        verbose_name=_(u"link target"),
        default=0,
        null=False,
        blank=False,
        choices=[(0, u"_self"), (1, u"_blank")]
    )

    priority = models.IntegerField(
        verbose_name=_(u"priority"),
        default=0,
        null=False,
        blank=False
    )

    @property
    def keys_list(self):
        return [ob.strip() for ob in self.keys.split("\n") if ob.strip()]

    @property
    def url(self):
        if self.direct_link:
            return self.direct_link
        return get_object(self.content_type, self.object_pk).get_absolute_url()

    def __unicode__(self):
        return self.keys

    class Meta(object):
        ordering = ["priority"]
        verbose_name = _(u"link")
        verbose_name_plural = _(u"links")



# coding: utf-8
from collections import OrderedDict
from django.http import HttpResponse, Http404
from django_relinking.content_types import objects_choices

import json


def get_objects(request):
    try:
        return HttpResponse(
            json.dumps(OrderedDict(objects_choices(request.POST['content_type']))),
            content_type="text/json; charset=utf8"
        )
    except IndexError:
        raise Http404



# coding: utf-8
from django import forms
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django_relinking.content_types import content_types_choices, objects_choices
from django_relinking.models import Link


class LinkForm(forms.ModelForm):

    model = Link

    def __init__(self, *a, **k):
        super(LinkForm, self).__init__(*a, **k)
        types_choices = content_types_choices()
        self.fields["content_type"].widget = forms.Select(
            choices=types_choices,
            attrs=dict(get_objects_url=reverse("get_content_type_objects"))
        )
        ctype = (
            self.data.get("content_type")
            or self.initial.get("content_type")
            or self.instance.content_type
            or (types_choices and types_choices[0][0])
        )
        choices = list(objects_choices(ctype) if ctype else [])

        is_object_required = -1 not in dict(choices)

        attrs = {}
        if not is_object_required:
            attrs["disabled"] = "disabled"
        self.fields["object_pk"].widget = forms.Select(choices=choices, attrs=attrs)
        self.fields["object_pk"].required = is_object_required

    def clean_content_type(self):
        content_type = self.cleaned_data.get("content_type")
        if content_type and self.cleaned_data.get("direct_link"):
            forms.ValidationError(_(u"This field is required."))
        return content_type

    def clean_object_pk(self):
        object_pk = self.cleaned_data.get("object_pk")
        if object and self.cleaned_data.get("direct_link"):
            forms.ValidationError(_(u"This field is required."))
        return object_pk

    def clean_direct_link(self):
        if not self.cleaned_data.get("direct_link") and not self.cleaned_data.get("content_type"):
            forms.ValidationError(_(u"This field is required."))
        return self.cleaned_data.get("direct_link")

    class Media(object):
        js = ("js/django-relinking.js",)



# coding: utf-8
from django.contrib import admin
from django_relinking.forms import LinkForm
from django_relinking.models import Link


class LinkAdmin(admin.ModelAdmin):

    search_fields = ['keys', 'content_type']
    list_display = ('keys', 'content_type', 'pk', 'direct_link')
    form = LinkForm


admin.site.register(Link, LinkAdmin)



# coding: utf-8
from django.conf import settings
from django_relinking.models import Link
from hashlib import md5


link_template = getattr(
    settings, "RELINKING_LINK_TEMPLATE",
    '<a target="{target}" href="{url}">{text}</a>'
)
index_pattern = getattr(
    settings, "RELINKING_INDEX_PATTERN",
    "<%=link {}=%>"
)
key_pattern = getattr(
    settings, "RELINKING_CACHE_PREFIX",
    "{links_table}.{hash}"
)
enable_cache = getattr(settings, "RELINKING_ENABLE_CACHE", False)


def get_relinked_text(origin):
    links = []
    for link in Link.objects.all():
        for key in link.keys_list:
            i = len(links)
            links.append(link_template.format(
                target=dict(Link.TARGET_CHOICES)[link.target],
                url=link.url,
                text=key
            ))
            origin = origin.replace(key, index_pattern.format(i))
    for i, link in enumerate(links):
        origin = origin.replace(index_pattern.format(i), link)
    return origin


def relink_text(origin):
    if enable_cache:
        from django.core.cache import cache
        key = key_pattern.format(
            links_table=Link._meta.db_table,
            hash=md5(origin).hexdigest()
        )
        relinked = cache.get(key, None)
        if relinked is None:
            relinked = get_relinked_text(origin)
            cache.set(key, relinked)
    else:
        relinked = get_relinked_text(origin)
    return relinked



# coding: utf-8
from django import template
from django.utils.safestring import mark_safe
from django_relinking import relink_text


register = template.Library()


@register.filter
def relink(origin):
    """
    Relink origin text
    """
    return mark_safe(relink_text(origin))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Link'
        db.create_table(u'django_relinking_link', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('keys', self.gf('django.db.models.fields.TextField')()),
            ('content_type', self.gf('django.db.models.fields.CharField')(default=None, max_length=255, null=True)),
            ('object_pk', self.gf('django.db.models.fields.CharField')(default=None, max_length=255, null=True)),
            ('direct_link', self.gf('django.db.models.fields.CharField')(default=None, max_length=255, null=True)),
            ('target', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('priority', self.gf('django.db.models.fields.IntegerField')(default=0)),
        ))
        db.send_create_signal(u'django_relinking', ['Link'])


    def backwards(self, orm):
        # Deleting model 'Link'
        db.delete_table(u'django_relinking_link')


    models = {
        u'django_relinking.link': {
            'Meta': {'ordering': "['priority']", 'object_name': 'Link'},
            'content_type': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True'}),
            'direct_link': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keys': ('django.db.models.fields.TextField', [], {}),
            'object_pk': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True'}),
            'priority': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'target': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        }
    }

    complete_apps = ['django_relinking']


# coding: utf-8

import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-relinking',
    version='0.0.1',
    packages=['django_relinking', 'django_relinking.migrations', 'django_relinking.templatetags'],
    install_requires=[
        'django>=1.4.5',
        'south>=0.7.6'
    ],
    include_package_data=True,
    license='BSD License',
    description='Provide relinking features.',
    long_description=README,
    url='http://django-relinking.beslave.net/',
    author='Vladislav Lozhechnik',
    author_email='lozhechnik.vladislav@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# coding: utf-8
from collections import OrderedDict
from django.db.models import Model


__CONTENT_TYPES__ = OrderedDict()


class ContentType(object):

    def __init__(self, cls, verbose_name, objects=None):
        self.cls = cls
        self.verbose_name = verbose_name
        self._objects = objects

    @property
    def objects(self):
        if hasattr(self._objects, "__call__"):
            return self._objects()
        elif self._objects is not None:
            return self._objects
        return None

    def get_object(self, pk=None):
        if issubclass(self.cls, Model):
            return self.objects.get(pk=pk)
        return self.cls


def register(content_type, verbose_name=None, objects=None):
    if verbose_name is None:
        if issubclass(content_type, Model):
            verbose_name = content_type._meta.verbose_name.title()
        elif hasattr(content_type, "verbose_name"):
            if hasattr(content_type.verbose_name, "__call__"):
                verbose_name = content_type.verbose_name().title()
            else:
                verbose_name = content_type.verbose_name.title()
        else:
            verbose_name = content_type.__name__
    __CONTENT_TYPES__[repr(content_type)] = ContentType(content_type, verbose_name, objects)


def unregister(content_type):
    if repr(content_type) in __CONTENT_TYPES__:
        del __CONTENT_TYPES__[repr(content_type)]


def register_all(*content_types):
    for content_type in content_types:
        register(content_type)


def unregister_all(*content_types):
    global __CONTENT_TYPES__
    if len(content_types) == 0:
        __CONTENT_TYPES__ = OrderedDict()
    else:
        for content_type in content_types:
            unregister(content_type)


def content_types_choices():
    choices = [(k, v.verbose_name) for k, v in __CONTENT_TYPES__.items()]
    return choices


def objects_choices(content_type):
    if content_type not in __CONTENT_TYPES__:
        raise IndexError
    ct = __CONTENT_TYPES__[content_type]
    objects = ct.objects
    if objects is None:
        return [(-1, u"'{}' hasn't objects".format(ct.verbose_name))]
    return ((ob.pk, unicode(ob)) for ob in objects)


def get_object(content_type, pk=None):
    return __CONTENT_TYPES__[content_type].get_object(pk)
